﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace StdDemo
{
    public partial class PDF417 : Form
    {
        private char[] data = new char[1];
        private long size = 1;
        private bool result;
        private int PatternRows, PatternCols;
        private int RectWidth, RectHeight;
        private int New_Rows, New_Cols;
        private int BarcodeWidth, BarcodeHeight;

        public PDF417()
        {
            InitializeComponent();
        }

        private char GetElement(ref char[] Data, int RowsCount, int ColsCount, int RowIndex, int ColIndex)
        {
            return Data[(RowIndex - 1) * ColsCount + ColIndex - 1];
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            printBarcode.Print();
        }

        private void printBarcode_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            GetPatternData(e.Graphics);
            Render(e.Graphics, 100, 150);
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Bitmap objBMP = new Bitmap(10, 10);
            Graphics g = Graphics.FromImage(objBMP);
            GetPatternData(g);
            objBMP.Dispose();

            SolidBrush sb = new SolidBrush(Color.White);
            const int PADDING_SIZE = 50;
            Rectangle rect = new Rectangle(0, 0, BarcodeWidth + PADDING_SIZE, BarcodeHeight + PADDING_SIZE);
            objBMP = new Bitmap(rect.Width, rect.Height);
            g = Graphics.FromImage(objBMP);
            g.FillRectangle(sb, rect);
            sb.Dispose();

            Render(g, PADDING_SIZE / 2, PADDING_SIZE / 2);
            objBMP.Save("c:\\temp\\PDF417.bmp", ImageFormat.Bmp);
            objBMP.Dispose();
        }

        private void GetPatternData(Graphics g)
        {
            // Message 
            objPDF417.Data = txtMessage.Text;

            // 0 for Binary Mode, 1 for Text Mode, 2 for Auto mode
            objPDF417.Mode = (MW6.Barcode.Std.PDF417.enumMode)cbxMode.SelectedIndex;

            // Error Correction Level (0 - 8)
            objPDF417.ErrorCorrectionLevel = (MW6.Barcode.Std.PDF417.enumECLevel)cbxECLevel.SelectedIndex;

            // PDF417 Rows (3 - 90)
            objPDF417.Rows = System.Convert.ToInt16(txtRows.Text);

            // PDF417 Columns (3 - 30)
            objPDF417.Columns = System.Convert.ToInt16(txtCols.Text);

            // Truncate symbol or not?
            objPDF417.TruncateSymbol = chkTruncateSymbol.Checked;

            // Handle tilde or not? 
            objPDF417.HandleTilde = chkHandleTilde.Checked;

            // Get the buffer size
            PatternRows = 0;
            PatternCols = 0;

            GraphicsUnit OldUnit = g.PageUnit;
            g.PageUnit = GraphicsUnit.Pixel;
            result = objPDF417.GetPatternData(ref data, ref size, ref PatternRows, ref PatternCols);
            if (!result)
            {
                // Create an array
                data = new char[size];

                // Obtain the pattern matrix data
                result = objPDF417.GetPatternData(ref data, ref size, ref PatternRows, ref PatternCols);
            }

            RectWidth = (int)(g.DpiX * (float)System.Convert.ToDouble(cbxNarrowBarWidth.Text) / 2.54);
            RectHeight = (int)(RectWidth * (float)System.Convert.ToDouble(cbxY2XRatio.Text));

            BarcodeWidth = PatternCols * RectWidth;
            BarcodeHeight = PatternRows * RectHeight;

            g.PageUnit = OldUnit;
        }

        private void Render(Graphics g, int StartX, int StartY)
        { 
            New_Rows = PatternRows;
            New_Cols = PatternCols;

            SolidBrush sb = new SolidBrush(Color.Black);

            int Start_X = StartX;
            int Start_Y = StartY;
            int X = Start_X;
            int Y = Start_Y;
            bool IsBar;
            int BarCount;
            
            for (int i = 1; i <= New_Rows; i++)
            {
                X = Start_X;
                IsBar = true;
                BarCount = 0;
                for (int j = 1; j <= New_Cols; j++)
                {
                    char OneElement = GetElement(ref data, New_Rows, New_Cols, i, j);
                    if (OneElement == '1' && IsBar)
                    {
                        // Another narrow bar
                        BarCount = BarCount + 1;
                    }
                    else if (OneElement == '1' && !IsBar)
                    {
                        // A narrow bar appears
                        IsBar = true;
                        BarCount = 1;
                    }
                    else if (OneElement == '0' && IsBar)
                    {
                        // Draw the continuous narrow bars
                        Rectangle rc = new Rectangle(X, Y, BarCount * RectWidth, RectHeight);

                        g.FillRectangle(sb, rc);
                        X = X + BarCount * RectWidth;
                        X = X + RectWidth;

                        IsBar = false;
                    }
                    else if (OneElement == '0' && !IsBar)
                    {
                        // A space (no drawing)
                        X = X + RectWidth;
                    }
                }

                if (IsBar)
                {
                    // Draw the continuous narrow bars
                    Rectangle rc = new Rectangle(X, Y, BarCount * RectWidth, RectHeight);

                    g.FillRectangle(sb, rc);
                }
                Y = Y + RectHeight;
            }

            sb.Dispose();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void PDF417_Load(object sender, EventArgs e)
        {
            cbxECLevel.SelectedIndex        = 2;
            cbxMode.SelectedIndex           = 0;
            cbxY2XRatio.SelectedIndex       = 0;
            cbxNarrowBarWidth.SelectedIndex = 2;
        }
    }
}
